package com.qmth.distributed.print.business.util;

import com.itextpdf.text.Document;
import com.itextpdf.text.pdf.PdfCopy;
import com.itextpdf.text.pdf.PdfImportedPage;
import com.itextpdf.text.pdf.PdfReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileOutputStream;
import java.util.Objects;

/**
 * @Description: 合并pdf util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/7
 */
public class PdfUtil {
    private final static Logger log = LoggerFactory.getLogger(PdfUtil.class);

    /**
     * 将多个PDF合并成一个PDF
     *
     * @param files
     * @param outputPath
     */
    public static void mergePdf(String[] files, String outputPath) {
        Document document = null;
        PdfCopy copy = null;
        PdfReader reader = null;
        try {
            document = new Document(new PdfReader(files[0]).getPageSize(1));
            copy = new PdfCopy(document, new FileOutputStream(outputPath));
            document.open();
            for (int i = 0; i < files.length; i++) {
                reader = new PdfReader(files[i]);
                int numberOfPages = reader.getNumberOfPages();
                for (int j = 1; j <= numberOfPages; j++) {
                    document.newPage();
                    PdfImportedPage page = copy.getImportedPage(reader, j);
                    copy.addPage(page);
                }
            }
        } catch (Exception e) {
            log.error("请求出错", e);
        } finally {
            if (Objects.nonNull(document)) {
                document.close();
            }
            if (Objects.nonNull(reader)) {
                reader.close();
            }
            if (Objects.nonNull(copy)) {
                copy.flush();
                copy.close();
            }
        }
    }

//    public static void main(String[] args) {
//        String[] files = {
//                "/Users/king/Downloads/测试附件/101_148382537591291904_signAndSticker.pdf",
//                "/Users/king/Downloads/pdf/3120ee49668e4bf48fd8c161a2abb73b的副本A3.pdf",
//                "/Users/king/Downloads/测试附件/101_158277997340655616_paperPackage.pdf"
////                "/Users/king/Downloads/测试附件/101_148382537591291904_signAndSticker.pdf"
//        };
//        String outputPath = "/Users/king/Downloads/pdf/merge1.pdf";
//        MergePdfUtil.mergePdf(files, outputPath);
//    }
}
